"""
Thin wrapper for backward compatibility.

This module re-exports from the crisprdesigner2 package.
Import directly from crisprdesigner2.scoring for new code.
"""

from crisprdesigner2._rs3_scoring import (
    score_guides_rs3,
    score_single_guide_rs3,
    encode_guide_rs3,
    load_rs3_model,
)

__all__ = [
    "score_guides_rs3",
    "score_single_guide_rs3",
    "encode_guide_rs3",
    "load_rs3_model",
]
