"""
Thin wrapper for backward compatibility.

This module re-exports from the crisprdesigner2 package.
Import directly from crisprdesigner2.filters for new code.
"""

from crisprdesigner2.filters import (
    apply_filters,
    get_passing_guides,
    is_valid_guide,
    FILTERS,
    _is_low_complexity,
    _pol3_termination,
    _high_tu_content,
    _tu_homopolymer,
    _mononucleotide_repeat,
    _low_complexity,
    _low_gc,
    _high_gc,
)

__all__ = [
    "apply_filters",
    "get_passing_guides",
    "is_valid_guide",
    "FILTERS",
    "_is_low_complexity",
    "_pol3_termination",
    "_high_tu_content",
    "_tu_homopolymer",
    "_mononucleotide_repeat",
    "_low_complexity",
    "_low_gc",
    "_high_gc",
]
