"""
Thin wrapper for backward compatibility.

This module re-exports from the crisprdesigner2 package.
Import directly from crisprdesigner2.output for new code.
"""

from crisprdesigner2.output import (
    write_outputs,
    read_design_guides_txt,
    read_design_guides_bed,
    scale_score_to_bed,
    validate_dataframe,
    TXT_COLUMNS,
    BED_COLUMNS,
)

__all__ = [
    "write_outputs",
    "read_design_guides_txt",
    "read_design_guides_bed",
    "scale_score_to_bed",
    "validate_dataframe",
    "TXT_COLUMNS",
    "BED_COLUMNS",
]
