"""
Thin wrapper for backward compatibility.

This module re-exports from the crisprdesigner2 package.
Import directly from crisprdesigner2.extraction for new code.
"""

from crisprdesigner2.extraction import (
    extract_guides,
    extract_guides_from_bed,
    open_fasta,
    reverse_complement,
    CONTEXT_PADDING,
)

__all__ = [
    "extract_guides",
    "extract_guides_from_bed",
    "open_fasta",
    "reverse_complement",
    "CONTEXT_PADDING",
]
